@@ -65,6 +65,7 @@
 
 #define OFFER_CALL(r2chan) \
 (r2chan)->call_state = OR2_CALL_OFFERED; \
+(r2chan)->caller_category = !((r2chan)->caller_category) ? (r2chan)->mf_read_tone : (r2chan)->caller_category; \
 EMI((r2chan))->on_call_offered((r2chan), (r2chan)->caller_ani_is_restricted ? NULL : (r2chan)->ani, (r2chan)->dnis, tone2category((r2chan)))
 
 static void r2config_argentina(openr2_context_t *r2context)
@@ -117,6 +118,20 @@ static void r2config_china(openr2_context_t *r2context)
 	r2context->mf_g1_tones.no_more_dnis_available = OR2_MF_TONE_INVALID;
 }
 
+static void r2config_indonesia(openr2_context_t *r2context)
+{
+	OR2_CONTEXT_STACK;
+	r2context->mf_ga_tones.request_category = OR2_MF_TONE_6;
+	r2context->mf_ga_tones.request_next_ani_digit = OR2_MF_TONE_6;
+	r2context->mf_ga_tones.request_all_dnis_again = OR2_MF_TONE_2;
+	r2context->mf_ga_tones.request_dnis_minus_1 = OR2_MF_TONE_8;
+	r2context->mf_ga_tones.request_dnis_minus_2 = OR2_MF_TONE_9;
+	r2context->mf_ga_tones.request_dnis_minus_3 = OR2_MF_TONE_INVALID;
+
+	r2context->mf_ga_tones.address_complete_charge_setup = OR2_MF_TONE_5;
+	return;
+}
+
 static void r2config_itu(openr2_context_t *r2context)
 {
 	OR2_CONTEXT_STACK;
@@ -264,6 +279,13 @@ static openr2_variant_entry_t r2variants[] =
 		.country = "Ecuador",
 		.config = r2config_itu,
 	},	
+	/* Indonesia */
+	{
+		.id = OR2_VAR_INDONESIA,
+		.name = "ID",
+		.country = "Indonesia",
+		.config = r2config_indonesia
+	},
 	/* ITU */
 	{
 		.id = OR2_VAR_ITU,
@@ -375,6 +397,7 @@ int openr2_proto_configure_context(openr2_context_t *r2context, openr2_variant_t
 
 	/* DTMF start dialing timer */
 	r2context->timers.dtmf_start_dial = 500;
+	r2context->timers.dtmf_detection_end = 5000;
 
 	/* Max ANI and DNIS */
 	r2context->max_dnis = (max_dnis >= OR2_MAX_DNIS) ? OR2_MAX_DNIS - 1 : max_dnis;
@@ -472,11 +495,16 @@ static const char *r2state2str(openr2_cas_state_t r2state)
 		return "Answer Received with MF Pending";
 	case OR2_CLEAR_FWD_TXD:
 		return "Clear Forward Transmitted";
+	case OR2_EXECUTING_DOUBLE_ANSWER:
+		return "Executing Double Answer";
+	case OR2_CLEAR_BACK_AFTER_CLEAR_FWD_RXD:
+		return "Clear Back After Clear Forward Received";
 	case OR2_BLOCKED:
 		return "Blocked";
-	default: 
-		return "*Unknown*";
+	case OR2_INVALID_STATE:
+		return "Invalid";
 	}
+	return "*Unknown*";
 }
 
 static const char *mfstate2str(openr2_mf_state_t mf_state)
@@ -518,10 +546,8 @@ static const char *mfstate2str(openr2_mf_state_t mf_state)
 
 	case OR2_MF_DETECTING_DTMF:
 		return "Detecting DTMF";
-
-	default:
-		return "*Unknown*";
 	}
+	return "*Unknown*";
 }
 
 OR2_EXPORT_SYMBOL
@@ -552,9 +578,8 @@ const char *openr2_proto_get_error(openr2_protocol_error_t error)
 		return "OpenR2 Library BUG";
 	case OR2_INTERNAL_ERROR:
 		return "OpenR2 Internal Error";
-	default:
-		return "*Unknown*";
 	}
+	return "*Unknown*";
 }
 
 static const char *mfgroup2str(openr2_mf_group_t mf_group)
@@ -586,10 +611,8 @@ static const char *mfgroup2str(openr2_mf_group_t mf_group)
 
 	case OR2_MF_DTMF_BACK_INIT:
 		return "Backward DTMF init";
-
-	default:
-		return "*Unknown*";
 	}
+	return "*Unknown*";
 }
 
 static const char *callstate2str(openr2_call_state_t state)
@@ -635,9 +658,8 @@ const char *openr2_proto_get_disconnect_string(openr2_call_disconnect_cause_t ca
 		return "Collect Call Rejected";
 	case OR2_CAUSE_FORCED_RELEASE:
 		return "Forced Release";
-	default:
-		return "*Unknown*";
 	}
+	return "*Unknown*";
 }
 
 static void close_logfile(openr2_chan_t *r2chan);
@@ -665,6 +687,7 @@ static void openr2_proto_init(openr2_chan_t *r2chan)
 	r2chan->category_sent = 0;
 	r2chan->mf_write_tone = 0;
 	r2chan->mf_read_tone = 0;
+	r2chan->dtmf_detection_done = 0;
 	openr2_set_flag(r2chan, OR2_CHAN_CALL_DNIS_CALLBACK);
 	close_logfile(r2chan);
 }
@@ -830,6 +853,12 @@ static void open_logfile(openr2_chan_t *r2chan, int backward)
 	}
 }
 
+static void dtmf_detection_expired(openr2_chan_t *r2chan)
+{
+	openr2_log(r2chan, OR2_LOG_DEBUG, "Stopping DTMF detection due to timer expiration\n");
+	r2chan->dtmf_detection_done = 1;
+}
+
 static void on_dtmf_received(void *user_data, const char *digits, int len)
 {
 	const char *digit = NULL;
@@ -839,14 +868,15 @@ static void on_dtmf_received(void *user_data, const char *digits, int len)
 		openr2_log(r2chan, OR2_LOG_ERROR, "Wow! DTMF detector gave us null digits of len %d\n", len);
 		return;
 	}
-	/* got a digit, reset silence counter */
-	r2chan->dtmf_silence_samples = 0; 
+	/* got a digit, reset dtmf end timer */
+	openr2_chan_cancel_timer(r2chan, &r2chan->timer_ids.dtmf_detection_end);
+
 	if (!openr2_test_flag(r2chan, OR2_CHAN_CALL_DNIS_CALLBACK)) {
 		openr2_log(r2chan, OR2_LOG_DEBUG, "Ignoring DNIS DTMF digits %s of len %d per user request\n", digits, len);
 		return;
 	}
 	openr2_log(r2chan, OR2_LOG_DEBUG, "Got digits %s of len %d\n", digits, len);
-	/* since we always read in 20ms chunks I dont think we can get more than 1 digit in a single chunk, may be even 2 chunks or more
+	/* since we always read in 20ms chunks I don't think we can get more than 1 digit in a single chunk, may be even 2 chunks or more
 	   are required to detect a single dtmf digit, but lets assume more than one can be received */
 	digit = digits;
 	/* check both len and digits to be more bug-safe from the DTMF detector implementation */
@@ -854,13 +884,21 @@ static void on_dtmf_received(void *user_data, const char *digits, int len)
 		r2chan->dnis[r2chan->dnis_len++] = *digit;
 		r2chan->dnis[r2chan->dnis_len] = '\0';
 		rc = EMI(r2chan)->on_dnis_digit_received(r2chan, *digit);
-		if (!rc) {
-			openr2_log(r2chan, OR2_LOG_DEBUG, "User requested us to stop getting DNIS!\n");
+		if (DNIS_COMPLETE(r2chan) || !rc) {
+			if (!rc) {
+				openr2_log(r2chan, OR2_LOG_DEBUG, "User requested us to stop getting DNIS!\n");
+			} else {
+				openr2_log(r2chan, OR2_LOG_DEBUG, "Done getting DNIS!\n");
+			}
 			openr2_clear_flag(r2chan, OR2_CHAN_CALL_DNIS_CALLBACK);
+			r2chan->dtmf_detection_done = 1;
+			return;
 		}
 		digit++;
 		len--;
 	}
+	r2chan->timer_ids.dtmf_detection_end = openr2_chan_add_timer(r2chan, TIMER(r2chan).dtmf_detection_end, 
+								dtmf_detection_expired, "dtmf_detection_end");
 }
 
 static void handle_incoming_call(openr2_chan_t *r2chan)
@@ -897,6 +935,8 @@ static void handle_incoming_call(openr2_chan_t *r2chan)
 		r2chan->mf_group = OR2_MF_DTMF_BACK_INIT;
 		r2chan->mf_state = OR2_MF_DETECTING_DTMF;
 		r2chan->detecting_dtmf = 1;
+		r2chan->timer_ids.dtmf_detection_end = openr2_chan_add_timer(r2chan, TIMER(r2chan).dtmf_detection_end, 
+									dtmf_detection_expired, "dtmf_detection_end");
 		openr2_log(r2chan, OR2_LOG_DEBUG, "Initialized R2 DTMF detector\n");
 	}
 	r2chan->r2_state = OR2_SEIZE_ACK_TXD;
@@ -946,7 +986,7 @@ static void prepare_mf_tone(openr2_chan_t *r2chan, int tone)
 			if (r2chan->direction == OR2_DIR_BACKWARD) {
 				/* schedule a new timer that will handle the timeout for our backward request */
 				r2chan->timer_ids.mf_back_cycle = openr2_chan_add_timer(r2chan, TIMER(r2chan).mf_back_cycle, 
-				mf_back_cycle_timeout_expired, "mf_back_cycle");
+									mf_back_cycle_timeout_expired, "mf_back_cycle");
 			}
 		}	
 		r2chan->mf_write_tone = tone;
@@ -1337,7 +1377,7 @@ handlecas:
 			CAS_LOG_RX(IDLE);
 			report_call_end(r2chan);
 		} else if (check_backward_disconnection(r2chan, cas, &out_disconnect_cause, &out_r2_state)) {
-			/* we requested the disconnection, we dont report call end to the user since the channel
+			/* we requested the disconnection, we don't report call end to the user since the channel
 			 * is still NOT available to be used, we need still to wait for IDLE
 			 * */
 			r2chan->r2_state = OR2_CLEAR_BACK_AFTER_CLEAR_FWD_RXD;
@@ -1417,7 +1457,7 @@ static int get_tone_from_mode(openr2_chan_t *r2chan, openr2_call_mode_t mode)
 	case OR2_CALL_SPECIAL:
 		return GB_TONE(r2chan).special_info_tone;
 	default:
-		openr2_log(r2chan, OR2_LOG_WARNING, "Unkown call mode (%d), defaulting to %s\n", get_string_from_mode(OR2_CALL_NO_CHARGE));
+		openr2_log(r2chan, OR2_LOG_WARNING, "Unknown call mode (%d), defaulting to %s\n", get_string_from_mode(OR2_CALL_NO_CHARGE));
 		return GB_TONE(r2chan).accept_call_no_charge;
 	}
 }
@@ -1984,7 +2024,7 @@ static void mf_send_ani(openr2_chan_t *r2chan)
 	/* TODO: Handle sending of previous ANI digits */
 	OR2_CHAN_STACK;
 
-	/* before trying to send, check if we already said we dont have more ANI */
+	/* before trying to send, check if we already said we don't have more ANI */
 	if (r2chan->mf_state == OR2_MF_ANI_END_TXD) {
 		/* this means probably that they are asking for something else, most likely DNIS
 		   if the ANI tone is the same as DNIS, then change to GA again to continue
@@ -2388,7 +2428,7 @@ int openr2_proto_make_call(openr2_chan_t *r2chan, const char *ani, const char *d
 		digit++;
 	}
 
-	/* open the log for the new call, but dont forget to close it if the call attempt fails here */
+	/* open the log for the new call, but don't forget to close it if the call attempt fails here */
 	open_logfile(r2chan, 0);
 
 	openr2_log(r2chan, OR2_LOG_DEBUG, "Outgoing call proceeding: ANI=%s, DNIS=%s, Category=%s\n", 
@@ -2514,7 +2554,7 @@ int openr2_proto_disconnect_call(openr2_chan_t *r2chan, openr2_call_disconnect_c
 	OR2_CHAN_STACK;
 	/* cannot drop a call when there is none to drop */
 	if (r2chan->call_state == OR2_CALL_IDLE) {
-		openr2_log(r2chan, OR2_LOG_ERROR, "Cannot disconnect call when we dont have a call to disconnect\n");
+		openr2_log(r2chan, OR2_LOG_ERROR, "Cannot disconnect call when we don't have a call to disconnect\n");
 		return -1;
 	}
 
